/* 
Name: OBJloader.h
Version: 1.0

Description: Header file for Wawefront OBJ file geometry loading
Author: Jiri Filip, UTIA AS CR, filipj'at'utia'dot'cas'dot'cz
Date: 28/05/07

COPYRIGHT ======================================================================

This source code is release under Creative Commons licence
(Attribution-NonCommercial 3.0 Unported) specified here:

http://creativecommons.org/licenses/by-nc/3.0/
http://creativecommons.org/licenses/by-nc/3.0/legalcode

You are free:
    * to Share - to copy, distribute and transmit the work
    * to Remix - to adapt the work
Under the following conditions:
    * Attribution - You must attribute the work in the manner specified by the
      author or licensor (but not in any way that suggests that they endorse you
      or your use of the work).

      Attribute this work:
      ------------------------------------------------
      Bidirectional Texture Function Compression based on Multi-Level Vector 
      Quantization
      as the paper written by
          Vlastimil Havran 	Jiri Filip 	Karol Myszkowski

      and published in Computer Graphics Forum, Volume 29, Issue 1, 
      pages 175-190, March 2010, DOI: 10.1111/j.1467-8659.2009.01585.x

      Project page:
        http://www.cgg.cvut.cz/members/havran/btfbase/
      ------------------------------------------------
     
    * Noncommercial - You may not use this work for commercial purposes.

With the understanding that:
    * Waiver - Any of the above conditions can be waived if you get permission 
      from the copyright holder.
    * Public Domain - Where the work or any of its elements is in the public 
      domain under applicable law, that status is in no way affected by 
      the license.
    * Other Rights - In no way are any of the following rights affected by the 
      license:
          o Your fair dealing or fair use rights, or other applicable copyright 
            exceptions and limitations;
          o The author's moral rights;
          o Rights other persons may have either in the work itself or in how 
            the work is used, such as publicity or privacy rights.
    * Notice - For any reuse or distribution, you must make clear to others 
         the license terms of this work. The best way to do this is with a link 
         to this web page.
===========================================================================  */

#ifndef objloader_c
#define objloader_c

#include <stdio.h>
#include <stdlib.h> 

class TObjectData{
 private:
  int dataSize;
  long i_v, i_vt, i_vn, i_f;
  char *fileName;
  char *OBJfileName;

 public:
  float **vertices;
  float **UVcoord;
  float **normals;
  long **faces;
  float **tangentsF;  // tangents for individual faces 
  float **tangentsV;  // tangents for individual vertices
  float **normalsF;   // normals for individual faces
  float **normalsV;   // normals for individual vertices

 public:
  TObjectData(int dataSize);
  ~TObjectData();
  
  int getNoOfTokens(char *linem, char *delimiter);
  
  //! loads geometry from OBJ file 
  int loadOBJfile(char *_fileName, int center, float scale, float xoff, float yoff, float zoff);

  //! gives total No of loaded faces
  long noOfFaces();
  //! gives total No of loaded vertices
  long noOfVertices();
  //! gives total No of loaded UVs
  long noOfUVcoord();
  //! gives total No of loaded normals
  long noOfNormals();
  
  //! computes tangents per face
  void tangentsPerFace();
  //! computes tangents per vertex
  void tangentsPerVertex();

  //! save computed tangents per vertex
  void saveTangentsPerVertex(char *fileName);
  //! load computed and already saved tangents per vertex
  bool loadTangentsPerVertex(char *fileName);

  //! get "vertex"th vertex coordinate "xyz" of face "face"
  float getV(int face, int vertex, int xyz);

  //! get "vertex"th vertex index of face "face"
  int getVindex(int face, int vertex);

  //! get "vertex"th normal coordinate "xyz" of face "face"
  float getN(int face, int vertex, int xyz);
  //! get "vertex"th interpolated normal coordinate "xyz" of face "face" - 3 normals for the face
  float getNv(int face, int vertex, int xyz);
  //! get normal coordinate "xyz" of face "face" - 1 normal for face 
  float getNf(int face, int xyz);

  //! get "vertex"th interpolated tangent coordinate "xyz" of face "face" - 3 tangents for the face
  float getT(int face, int vertex, int xyz);
  //! get tangent coordinate "xyz" of face "face" - 1 tangent for face 
  float getTf(int face, int xyz);

  //! get "vertex"th UV texture coordinate "uv" of face "face" - 3 UVs for face
  float getUV(int face, int vertex, int uv);

  //! basic vector operations in 3D
  void vec_crossprod(float *x, float *y, float *res);
  float vec_len(float *x);
  void vec_diff(float *x, float *y, float *res);
  int vec_norm(float *x);
  void normal(float *v1, float *v2, float *v3, float *n);

};

#endif
